@@ -1,32 +1,31 @@
+package JIRA::Client;
+# ABSTRACT: (DEPRECATED) Extended interface to JIRA's SOAP API
+$JIRA::Client::VERSION = '0.43';
 use strict;
 use warnings;
 
-package JIRA::Client;
-{
-  $JIRA::Client::VERSION = '0.40';
-}
-# ABSTRACT: An extended interface to JIRA's SOAP API.
-
 use Carp;
 use Data::Util qw(:check);
 use SOAP::Lite;
 
 
 sub new {
-    my $class = shift;
+    my ($class, @args) = @_;
 
     my $args;
 
-    if (@_ == 1) {
-	$args = shift;
+    if (@args == 1) {
+	$args = shift @args;
 	is_hash_ref($args) or croak "$class::new sole argument must be a hash-ref.\n";
 	foreach my $arg (qw/baseurl user password/) {
 	    exists $args->{$arg}
 		or croak "Missing $arg key to $class::new hash argument.\n";
 	}
 	$args->{soapargs} = [] unless exists $args->{soapargs};
-    } elsif (@_ >= 3) {
-	my ($baseurl, $user, $password, @args) = @_;
+    } elsif (@args >= 3) {
+        my $baseurl  = shift @args;
+        my $user     = shift @args;
+        my $password = shift @args;
 	$args = {
 	    baseurl  => $baseurl,
 	    user     => $user,
@@ -314,6 +313,8 @@ sub _flaten_components_and_versions {
     foreach my $field (grep {exists $params->{$_}} keys %JRA12300) {
 	$params->{$JRA12300{$field}} = delete $params->{$field};
     }
+
+    return;
 }
 
 
@@ -604,6 +605,7 @@ sub attach_files_to_issue {
 	    open my $fh, '<:raw', $file
 		or croak "Can't open $file: $!\n";
 	    push @attachments, $fh;
+            close $fh;
 	} elsif (is_hash_ref($file)) {
 	    while (my ($name, $contents) = each %$file) {
 		push @filenames, $name;
@@ -611,6 +613,7 @@ sub attach_files_to_issue {
 		    open my $fh, '<:raw', $contents
 			or croak "Can't open $contents: $!\n";
 		    push @attachments, $fh;
+                    close $fh;
 		} elsif (is_glob_ref($contents)
 			     || is_instance($contents => 'IO::File')
 				 || is_instance($contents => 'FileHandle')) {
@@ -661,38 +664,56 @@ sub attach_strings_to_issue {
 }
 
 
-sub filter_issues {
+sub filter_issues_unsorted {
     my ($self, $filter, $limit) = @_;
 
     $filter =~ s/^\s*"?//;
     $filter =~ s/"?\s*$//;
 
-    my $issues = do {
-	if ($filter =~ /^(?:[A-Z]+-\d+\s+)*[A-Z]+-\d+$/i) {
-	    # space separated key list
-	    [map {$self->getIssue(uc $_)} split / /, $filter];
-	} elsif ($filter =~ /^[\w-]+$/i) {
-	    # saved filter
-	    $self->getIssuesFromFilterWithLimit($filter, 0, $limit || 1000);
-	} else {
-	    # JQL filter
-	    $self->getIssuesFromJqlSearch($filter, $limit || 1000);
-	}
-    };
+    if ($filter =~ /^(?:[A-Z]+-\d+\s+)*[A-Z]+-\d+$/i) {
+        # space separated key list
+
+        # Let's construct a JQL query in the form "issuekey IN (...)" to
+        # pass to getIssuesFromJqlSearch.
+
+        my %keys = map {($_ => undef)} split / /, $filter; # discard duplicates
+        my @keys = keys %keys;
+
+        # If the list is too big we split it up and invoke
+        # getIssuesFromJqlSearch several times.
+        my @issues;
+        while (my @subkeys = splice(@keys, 0, 128)) {
+            my $jql = 'issuekey IN (' . join(',', @subkeys) . ')';
+            push @issues, @{$self->getIssuesFromJqlSearch($jql, 1000)};
+        }
+        return @issues;
+    } elsif ($filter =~ /^[\w-]+$/i) {
+        # saved filter
+        return @{$self->getIssuesFromFilterWithLimit($filter, 0, $limit || 1000)};
+    } else {
+        # JQL filter
+        return @{$self->getIssuesFromJqlSearch($filter, $limit || 1000)};
+    }
+}
+
+
+sub filter_issues {
+    my ($self, $filter, $limit) = @_;
 
     # Order the issues by project key and then by numeric value using
     # a Schwartzian transform.
-    map  {$_->[2]}
-	sort {$a->[0] cmp $b->[0] or $a->[1] <=> $b->[1]}
-	    map  {my ($p, $n) = ($_->{key} =~ /([A-Z]+)-(\d+)/); [$p, $n, $_]} @$issues;
+    return
+        map  {$_->[2]}
+            sort {$a->[0] cmp $b->[0] or $a->[1] <=> $b->[1]}
+                map  {my ($p, $n) = ($_->{key} =~ /([A-Z]+)-(\d+)/); [$p, $n, $_]}
+                    filter_issues_unsorted($self, $filter, $limit);
 }
 
 
+## no critic (Modules::ProhibitMultiplePackages)
+
 package RemoteFieldValue;
-{
-  $RemoteFieldValue::VERSION = '0.40';
-}
-
+$RemoteFieldValue::VERSION = '0.43';
 sub new {
     my ($class, $id, $values) = @_;
 
@@ -705,10 +726,7 @@ sub new {
 
 
 package RemoteCustomFieldValue;
-{
-  $RemoteCustomFieldValue::VERSION = '0.40';
-}
-
+$RemoteCustomFieldValue::VERSION = '0.43';
 sub new {
     my ($class, $id, $values) = @_;
 
@@ -718,10 +736,7 @@ sub new {
 
 
 package RemoteComponent;
-{
-  $RemoteComponent::VERSION = '0.40';
-}
-
+$RemoteComponent::VERSION = '0.43';
 sub new {
     my ($class, $id, $name) = @_;
     my $o = bless({id => $id}, $class);
@@ -731,10 +746,7 @@ sub new {
 
 
 package RemoteVersion;
-{
-  $RemoteVersion::VERSION = '0.40';
-}
-
+$RemoteVersion::VERSION = '0.43';
 sub new {
     my ($class, $id, $name) = @_;
     my $o = bless({id => $id}, $class);
@@ -969,15 +981,18 @@ sub AUTOLOAD {
 1; # End of JIRA::Client
 
 __END__
+
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
-JIRA::Client - An extended interface to JIRA's SOAP API.
+JIRA::Client - (DEPRECATED) Extended interface to JIRA's SOAP API
 
 =head1 VERSION
 
-version 0.40
+version 0.43
 
 =head1 SYNOPSIS
 
@@ -1007,6 +1022,11 @@ version 0.40
 
 =head1 DESCRIPTION
 
+B<DEPRECATION WARNING>: Please, before using this module consider using the
+newer L<JIRA::REST> because JIRA's SOAP API was
+L<deprecated|https://developer.atlassian.com/display/JIRADEV/SOAP+and+XML-RPC+API+Deprecated+in+JIRA+6.0>
+on JIRA 6.0 and won't be available anymore on JIRA 8.0.
+
 JIRA is a proprietary bug tracking system from Atlassian
 (L<http://www.atlassian.com/software/jira/>).
 
@@ -1382,7 +1402,7 @@ file names and the values their contents.
 The method retuns the value returned by the
 B<addBase64EncodedAttachmentsToIssue> API method.
 
-=head2 B<filter_issues> FILTER [, LIMIT]
+=head2 B<filter_issues_unsorted> FILTER [, LIMIT]
 
 This method returns a list of RemoteIssue objects from the specified
 FILTER, which is a string that is understood in one of these ways (in
@@ -1398,6 +1418,10 @@ to getIssue. For example:
 
     KEY-123 chave-234 CLAVE-345
 
+Note that the result list doesn't respect the order in which the keys are
+specified and also that duplicate keys are discarded and the corresponding
+issue appear only once in the resulting list.
+
 =item The name of a saved filter
 
 If FILTER is a single word, it is passed to
@@ -1423,7 +1447,10 @@ beings to request a list of issues. Be warned, however, that you are
 responsible to de-taint the FILTER argument before passing it to the
 method.
 
-The returned list of RemoteIssue objects is sorted by issue key.
+=head2 B<filter_issues> FILTER [, LIMIT]
+
+This method invokes the B<filter_issues_unsorted> method with the same
+arguments and returns the list of RemoteIssue objects sorted by issue key.
 
 =head1 OTHER CONSTRUCTORS
 
@@ -1474,16 +1501,27 @@ A scalar or an array of scalars.
 Please, see the examples under the C<examples> directory in the module
 distribution.
 
+=head1 SEE ALSO
+
+=over
+
+=item * L<JIRA::REST>
+
+=back
+
+=head1 REPOSITORY
+
+L<https://github.com/gnustavo/JIRA-Client>
+
 =head1 AUTHOR
 
 Gustavo L. de M. Chaves <gnustavo@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2012 by CPqD.
+This software is copyright (c) 2015 by CPqD.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
 =cut
-
